﻿' Copyright (C) Kentaro HARA 2012. All rights reserved.
Public Class Form1
    ' 宣言セクション
    Dim DataSize As Integer     ' サウンド データのサイズ。
    Dim DataOffset As Integer   ' ファイルの先頭からサウンド データまでのオフセット。
    Dim WaveFile As Integer     ' 開いた WAVE ファイルのファイル番号。
    Dim Buffer() As Byte        ' 再生時に使用するバッファー。

    ' WAVE ファイルを開く
    Private Sub btnOpen_Click(sender As System.Object, e As System.EventArgs) Handles btnOpen.Click
        Dim strFileName As String

        ' [ファイルを開く] ダイアログを表示します。
        If (openFileDialog.ShowDialog() = Windows.Forms.DialogResult.OK) Then
            strFileName = openFileDialog.FileName

            ' 初期化します。
            Init()

            ' ファイルを開きます。
            If (OpenWaveFile(strFileName) = True) Then
                ' [再生] ボタンを有効にします。
                btnPlay.Enabled = True

                ' [再生] ボタンにフォーカスを設定します。
                btnPlay.Focus()
            End If
        End If
    End Sub

    ' 再生を開始する
    Private Sub btnPlay_Click(sender As System.Object, e As System.EventArgs) Handles btnPlay.Click
        Dim i As Short
        Dim SizeToPlay As Integer

        ' WAVE データの先頭にシークします。
        Seek(WaveFile, DataOffset)

        ' デバイスを開きます。
        waveOut.Open()

        ' 再生を一時停止しておきます。
        waveOut.Pause()

        For i = 0 To waveOut.NumBuffers - 1
            ' 再生するデータ サイズを計算します。
            SizeToPlay = Math.Min(DataSize - (Seek(WaveFile) - DataOffset), waveOut.BufferSize)

            ' サイズが 0 以下ならば (データがなければ)、ループを抜けます。
            If (SizeToPlay <= 0) Then
                Exit For
            End If

            ' データ読み込み用バッファーのサイズを調整します。
            ReDim Buffer(SizeToPlay - 1)

            ' データを読み込みます。
            FileGet(WaveFile, Buffer)

            ' デバイスにデータを書き込みます。
            waveOut.SetData(Buffer, i)
            waveOut.AddBuffer(i, SizeToPlay)

            ' ファイルの最後まで来ていたら、ループを抜けます。
            If (EOF(WaveFile)) Then
                Exit For
            End If
        Next i

        ' 再生を開始します。
        waveOut.Restart()

        ' [停止] ボタンにフォーカスを設定します。
        btnStop.Focus()
    End Sub

    ' 再生を停止する
    Private Sub btnStop_Click(sender As System.Object, e As System.EventArgs) Handles btnStop.Click
        ' 再生を停止します。
        waveOut.Stop()
    End Sub

    ' WAVE ファイルを開く関数
    Private Function OpenWaveFile(strFileName As String) As Boolean
        ' WAVE ファイルを開きます。
        ' strFileName : 開く WAVE ファイル名。
        ' 戻り値      : 成功した場合は True、失敗した場合は False。

        Dim hdr As WAVEFILEHEADER

        ' 空いているファイル番号を取得します。
        WaveFile = FreeFile()

        ' バイナリ読み取りモードでファイルを開きます。
        FileOpen(WaveFile, strFileName, OpenMode.Binary, OpenAccess.Read, OpenShare.LockWrite)

        ' ヘッダ情報を取得します。
        FileGet(WaveFile, hdr)

        ' ファイルのフォーマットをチェックします。
        If (hdr.ckidRIFF <> "RIFF") Then
            ' エラー メッセージを表示し、ファイルを閉じます。
            MsgBox("このファイルは RIFF ファイルではありません。")
            CloseWaveFile()
            OpenWaveFile = False
            Exit Function
        End If

        If (hdr.fccType <> "WAVE") Then
            ' エラー メッセージを表示し、ファイルを閉じます。
            MsgBox("このファイルは WAVE ファイルではありません。")
            CloseWaveFile()
            OpenWaveFile = False
            Exit Function
        End If

        If (hdr.ckidFmt <> "fmt ") Then
            ' エラー メッセージを表示し、ファイルを閉じます。
            MsgBox("このファイルにはフォーマット情報がありません。")
            CloseWaveFile()
            OpenWaveFile = False
            Exit Function
        End If

        If (hdr.ckidData <> "data") Then
            ' エラー メッセージを表示し、ファイルを閉じます。
            MsgBox("このファイルにはサウンド データがありません。")
            CloseWaveFile()
            OpenWaveFile = False
            Exit Function
        End If

        If (hdr.WaveFmt.wf.wFormatTag <> WAVE_FORMAT_PCM) Then
            ' エラー メッセージを表示し、ファイルを閉じます。
            MsgBox("この WAVE ファイルは PCM 形式ではありません。")
            CloseWaveFile()
            OpenWaveFile = False
            Exit Function
        End If

        ' WaveOut コントロールのプロパティを設定します。
        With waveOut
            .Channels = hdr.WaveFmt.wf.nChannels
            .SamplesPerSec = hdr.WaveFmt.wf.nSamplesPerSec
            .BitsPerSample = hdr.WaveFmt.wBitsPerSample
        End With

        ' サウンド データのサイズを保存します。
        DataSize = hdr.ckSizeData

        ' ファイルの先頭からサウンド データまでのオフセットを保存します。
        DataOffset = Seek(WaveFile)

        OpenWaveFile = True
    End Function

    ' WAVE ファイルを閉じる
    Private Sub CloseWaveFile()
        ' WAVE ファイルを閉じます。
        If (WaveFile <> 0) Then
            FileClose(WaveFile)
            WaveFile = 0
        End If
    End Sub

    ' 初期化
    Private Sub Init()
        ' 初期化を行います。
        ' [再生]、[停止] ボタンを無効にします。
        btnPlay.Enabled = False
        btnStop.Enabled = False

        ' [開く] ボタンを有効にします。
        btnOpen.Enabled = True

        ' ファイルを閉じます。
        CloseWaveFile()

        ' 変数を初期化します。
        DataSize = 0
        DataOffset = 0
        Erase Buffer
    End Sub

    ' 終了処理
    Private Sub Form1_FormClosed(sender As System.Object, e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        ' 再生を停止します。
        waveOut.Stop()

        ' デバイスを閉じます。
        waveOut.Close()

        ' ファイルを閉じます。
        CloseWaveFile()
    End Sub

    ' Form_Load イベントの処理
    Private Sub Form1_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        ' 初期化します。
        Init()
    End Sub

    ' OnBusyFlagChange イベントの処理
    Private Sub waveOut_OnBusyFlagChange(sender As System.Object, e As System.EventArgs) Handles waveOut.OnBusyFlagChange
        If (waveOut.IsBusy) Then
            ' 再生中であれば、
            ' [再生]、[開く] ボタンを無効にします。
            btnPlay.Enabled = False
            btnOpen.Enabled = False

            ' [停止] ボタンを有効にします。
            btnStop.Enabled = True
        Else
            ' 再生中でなければ、
            ' WAVE 出力デバイスを閉じます。
            waveOut.Close()

            ' [再生]、[開く] ボタンを有効にします。
            btnPlay.Enabled = True
            btnOpen.Enabled = True

            ' [再生] ボタンにフォーカスを設定します。
            btnPlay.Focus()

            ' [停止] ボタンを無効にします。
            btnStop.Enabled = False
        End If
    End Sub

    ' OnDone イベントの処理
    Private Sub waveOut_OnDone(sender As System.Object, e As AxWaveInterfaceLib._DWaveOutEvents_OnDoneEvent) Handles waveOut.OnDone
        Dim SizeToPlay As Integer

        ' Stop メソッドが実行されていなければ、
        If (e.bStopped = False) Then
            ' すでにファイルの最後まで達していたら、何も処理しません。
            If (EOF(WaveFile)) Then
                Exit Sub
            End If

            ' 再生するデータ サイズを計算します。
            SizeToPlay = Math.Min(DataSize - (Seek(WaveFile) - DataOffset), waveOut.BufferSize)

            ' サイズが 0 以下ならば (データがなければ)、リターンします。
            If (SizeToPlay <= 0) Then
                Exit Sub
            End If

            ' データ読み込み用バッファーのサイズを調整します。
            ReDim Buffer(SizeToPlay - 1)

            ' データを読み込みます。
            FileGet(WaveFile, Buffer)

            ' デバイスにデータを追加します。
            waveOut.SetData(Buffer, e.bufferIndex)
            waveOut.AddBuffer(e.bufferIndex, SizeToPlay)
        End If
    End Sub
End Class
